#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @FileName  :data_nwp_ftp.py
# @Time      :2024/12/26 08:38
# @Author    :David
# @Company: shenyang JY
from ftplib import FTP
import pandas as pd
from flask import Flask,request
import time, traceback, io
from common.database_dml import insert_data_into_mongo
from common.alert import send_message
from common.logs import Log
from datetime import datetime, timedelta
logger = Log('data-processing').logger

app = Flask('data_tj_nwp_ftp——service')


def connect_ftp_get_data(params):
    date_obj = datetime.strptime(params['dt'], '%Y%m%d')
    # 减 1 天
    date_file = (date_obj - timedelta(days=1)).strftime('%Y%m%d')
    # 加 1 天
    d1 = (date_obj + timedelta(days=1)).strftime('%Y-%m-%d')
    farm_id = params['farmId']
    ftp_host = '58.240.173.20'  # FTP 服务器地址
    ftp_port = 65321  # FTP 服务器端口，默认为 21
    ftp_user = 'zktjcE'  # FTP 用户名
    ftp_pass = 'HAFuapDVyYV'  # FTP 密码
    ftp_directory = f'/contract/{date_file}12'  # 目标文件夹路径
    farm_csv = f'{farm_id}.csv'
    ftp = FTP()
    ftp.connect(ftp_host, ftp_port)  # 连接到指定的 FTP 服务器和端口
    ftp.login(ftp_user, ftp_pass)  # 使用用户名和密码登录
    ftp.cwd(ftp_directory)
    # 列出该文件夹中的所有文件
    files = ftp.nlst()
    if farm_csv not in files:
        send_message('FTP获取天机NWP数据接口组件', farm_id, '缺少NWP文件！')
    else:
        # 从 FTP 下载文件并读取到内存
        with io.BytesIO() as byte_io:
            ftp.retrbinary(f'RETR {farm_csv}', byte_io.write)  # 下载文件内容到内存
            byte_io.seek(0)  # 将文件指针定位到文件开头
            df = pd.read_csv(byte_io)  # 将 CSV 数据读取到 Pandas DataFrame
    # 关闭 FTP 连接
    ftp.quit()
    df['dateTime'] = pd.to_datetime(df['time(UTC)']).dt.tz_localize('UTC').dt.tz_convert('Asia/Shanghai').dt.strftime(
        '%Y-%m-%d %H:%M:%S')
    return df[df['dateTime']>=d1]

@app.route('/data_tj_nwp_ftp', methods=['POST'])
def data_tj_nwp_ftp():
    # 获取程序开始时间
    start_time = time.time()
    result = {}
    success = 0
    args = {}
    try:
        args = request.values.to_dict()
        logger.info(args)
        df = connect_ftp_get_data(args)
        insert_data_into_mongo(df, args)
        success = 1
    except Exception as e:
        my_exception = traceback.format_exc()
        my_exception.replace("\n", "\t")
        result['msg'] = my_exception
        logger.info("生产，获取原始天机nwp出错：{}".format(my_exception))
    end_time = time.time()
    result['success'] = success
    result['args'] = args
    result['start_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(start_time))
    result['end_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(end_time))
    return result


if __name__ == "__main__":
    print("Program starts execution!")
    from waitress import serve
    serve(app, host="0.0.0.0", port=10106)
    print("server start!")