import pandas as pd
from pymongo import MongoClient
import requests
from flask import Flask,request
import time
import logging
import traceback
from common.database_dml import get_data_from_mongo,insert_data_into_mongo
app = Flask('processing_limit_power_by_records——service')
    

def limit_record_judgement(power,args):
    timeBegin,timeEnd,station_code,col_time = args['timeBegin'], args['timeEnd'], args['station_code'], args['col_time']
    url = f'http://itil.jiayuepowertech.com:9958/itil/api/power-limitation?timeBegin={timeBegin}&timeEnd={timeEnd}'
    try:
        res = requests.get(url).json()
    except requests.RequestException as e:
        print(f"请求失败: {e}")
        return []
    ele_info = [item for item in res.get('data', []) if item['stationCode'] == station_code]
    initial_count = len(power)
    clean_record = []
    for ele in ele_info:
        begin,end = ele['timeBegin'], ele['timeEnd']
        ele_limits = pd.date_range(start=begin, end=end, freq='min').strftime("%Y-%m-%d %H:%M:%S").tolist()
        if len(ele_limits) > 0:
            clean_record = clean_record + ele_limits
    print(f"根据限电记录清洗，{timeBegin}至{timeEnd}，限电时长共计{round(len(clean_record)/60,2)}小时")
    return power[~power[col_time].isin(clean_record)]


@app.route('/processing_limit_power_by_records', methods=['POST'])
def processing_limit_power_by_records():
    # 获取程序开始时间  
    start_time = time.time()  
    result = {}
    success = 0
    print("Program starts execution!")
    try:
        args = request.values.to_dict()
        print('args',args)
        logger.info(args)
        power_df = get_data_from_mongo(args)
        res_df = limit_record_judgement(power_df,args)
        insert_data_into_mongo(res_df,args)
        success = 1
    except Exception as e:
        my_exception = traceback.format_exc()
        my_exception.replace("\n","\t")
        result['msg'] = my_exception
    end_time = time.time() 
   
    result['success'] = success
    result['args'] = args
    result['start_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(start_time))
    result['end_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(end_time))
    print("Program execution ends!")
    return result


if __name__=="__main__":
    print("Program starts execution!")
    logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    logger = logging.getLogger("processing_limit_power_by_records log")
    from waitress import serve
    serve(app, host="0.0.0.0", port=10088)
    print("server start!")