#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @FileName  :tf_bp_pre.py
# @Time      :2025/2/13 13:35
# @Author    :David
# @Company: shenyang JY
import json, copy
import numpy as np
from flask import Flask, request, g
import logging, argparse, traceback
from common.database_dml_koi import *
from common.processing_data_common import missing_features, str_to_list
from data_processing.data_operation.data_handler import DataHandler
from threading import Lock
import time, yaml
model_lock = Lock()
from itertools import chain
from common.logs import Log
from tf_bp import BPHandler
logger = Log('tf_bp').logger
np.random.seed(42)  # NumPy随机种子
# tf.set_random_seed(42)  # TensorFlow随机种子
app = Flask('tf_bp_pre——service')

with app.app_context():
    current_dir = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(current_dir, 'bp.yaml'), 'r', encoding='utf-8') as f:
        args = yaml.safe_load(f)
    dh = DataHandler(logger, args)
    bp = BPHandler(logger, args)
    global opt

@app.before_request
def update_config():
    # ------------ 整理参数，整合请求参数 ------------
    args_dict = request.values.to_dict()
    args_dict['features'] = args_dict['features'].split(',')
    args.update(args_dict)
    opt = argparse.Namespace(**args)
    dh.opt = opt
    bp.opt = opt
    g.opt = opt
    logger.info(args)

@app.route('/nn_bp_predict', methods=['POST'])
def model_prediction_bp():
    # 获取程序开始时间
    start_time = time.time()
    result = {}
    success = 0
    print("Program starts execution!")
    try:
        # ------------ 获取数据，预处理预测数据------------
        pre_data = get_data_from_mongo(args)
        feature_scaler, target_scaler = get_scaler_model_from_mongo(args)
        scaled_pre_x, pre_data = dh.pre_data_handler(pre_data, feature_scaler, bp_data=True)
        bp.opt.cap = round(target_scaler.transform(np.array([[args['cap']]]))[0, 0], 2)
        # ------------ 获取模型，预测结果------------
        bp.get_model(args)
        res = list(chain.from_iterable(target_scaler.inverse_transform(bp.predict(scaled_pre_x))))
        pre_data['farm_id'] = args.get('farm_id', 'null')
        if args.get('algorithm_test', 0):
            pre_data[args['model_name']] = res[:len(pre_data)]
            pre_data.rename(columns={args['col_time']: 'dateTime'}, inplace=True)
            pre_data = pre_data[['dateTime', 'farm_id', args['target'], args['model_name'], 'dq']]
            pre_data = pre_data.melt(id_vars=['dateTime', 'farm_id', args['target']], var_name='model', value_name='power_forecast')
            res_cols = ['dateTime', 'power_forecast', 'farm_id', args['target'], 'model']
        else:
            pre_data['cdq'] = args.get('cdq', 1)
            pre_data['dq'] = args.get('dq', 1)
            pre_data['zq'] = args.get('zq', 1)
            pre_data['power_forecast'] = res[:len(pre_data)]
            pre_data.rename(columns={args['col_time']: 'date_time'}, inplace=True)
            res_cols = ['date_time', 'power_forecast', 'farm_id', 'cdq', 'dq', 'zq']

        pre_data = pre_data[res_cols]

        pre_data['power_forecast'] = pre_data['power_forecast'].round(2)
        pre_data.loc[pre_data['power_forecast'] > args['cap'], 'power_forecast'] = args['cap']
        pre_data.loc[pre_data['power_forecast'] < 0, 'power_forecast'] = 0

        insert_data_into_mongo(pre_data, args)
        success = 1
    except Exception as e:
        my_exception = traceback.format_exc()
        my_exception.replace("\n", "\t")
        result['msg'] = my_exception
    end_time = time.time()

    result['success'] = success
    result['args'] = args
    result['start_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(start_time))
    result['end_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(end_time))
    print("Program execution ends!")
    return result


if __name__ == "__main__":
    print("Program starts execution!")
    from waitress import serve
    serve(app, host="0.0.0.0", port=10110)
    print("server start!")

    # ------------------------测试代码------------------------
    # args_dict = {"mongodb_database": 'david_test', 'scaler_table': 'j00083_scaler', 'model_name': 'bp1.0.test',
    #              'model_table': 'j00083_model', 'mongodb_read_table': 'j00083_test', 'col_time': 'date_time', 'mongodb_write_table': 'j00083_rs',
    #              'features': 'speed10,direction10,speed30,direction30,speed50,direction50,speed70,direction70,speed90,direction90,speed110,direction110,speed150,direction150,speed170,direction170'}
    # args_dict['features'] = args_dict['features'].split(',')
    # arguments.update(args_dict)
    # dh = DataHandler(logger, arguments)
    # bp = BPHandler(logger)
    # opt = argparse.Namespace(**arguments)
    #
    # opt.Model['input_size'] = len(opt.features)
    # pre_data = get_data_from_mongo(args_dict)
    # feature_scaler, target_scaler = get_scaler_model_from_mongo(arguments)
    # pre_x = dh.pre_data_handler(pre_data, feature_scaler, opt, bp_data=True)
    # bp.get_model(arguments)
    # result = bp.predict(pre_x)
    # result1 = list(chain.from_iterable(target_scaler.inverse_transform([result.flatten()])))
    # pre_data['power_forecast'] = result1[:len(pre_data)]
    # pre_data['farm_id'] = 'J00083'
    # pre_data['cdq'] = 1
    # pre_data['dq'] = 1
    # pre_data['zq'] = 1
    # pre_data.rename(columns={arguments['col_time']: 'date_time'}, inplace=True)
    # pre_data = pre_data[['date_time', 'power_forecast', 'farm_id', 'cdq', 'dq', 'zq']]
    #
    # pre_data['power_forecast'] = pre_data['power_forecast'].round(2)
    # pre_data.loc[pre_data['power_forecast'] > opt.cap, 'power_forecast'] = opt.cap
    # pre_data.loc[pre_data['power_forecast'] < 0, 'power_forecast'] = 0
    #
    # insert_data_into_mongo(pre_data, arguments)