#!/usr/bin/env python
# -*- coding: utf-8 -*-
# time: 2024/5/6 13:25
# file: time_series.py
# author: David
# company: shenyang JY
import json, copy
import numpy as np
from flask import Flask, request
import traceback
import logging, argparse
from data_processing.data_operation.data_handler import DataHandler
import time, yaml
from models_processing.model_tf.tf_cnn import CNNHandler
from common.database_dml_koi import *
import matplotlib.pyplot as plt
from common.logs import Log
# logger = logging.getLogger()
logger = Log('tf_cnn').logger
np.random.seed(42)  # NumPy随机种子
app = Flask('tf_cnn_train——service')

with app.app_context():
    current_dir = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(current_dir, 'cnn.yaml'), 'r', encoding='utf-8') as f:
        args = yaml.safe_load(f)

    dh = DataHandler(logger, args)
    cnn = CNNHandler(logger, args)

@app.before_request
def update_config():
    # ------------ 整理参数，整合请求参数 ------------
    args_dict = request.values.to_dict()
    args_dict['features'] = args_dict['features'].split(',')
    args.update(args_dict)
    opt = argparse.Namespace(**args)
    dh.opt = opt
    cnn.opt = opt
    logger.info(args)

@app.route('/nn_cnn_training', methods=['POST'])
def model_training_bp():
    # 获取程序开始时间
    start_time = time.time()
    result = {}
    success = 0
    print("Program starts execution!")
    try:
        # ------------ 获取数据，预处理训练数据 ------------
        train_data = get_data_from_mongo(args)
        train_x, train_y, valid_x, valid_y, scaled_train_bytes, scaled_target_bytes, scaled_cap = dh.train_data_handler(train_data)
        # ------------ 训练模型，保存模型 ------------
        cnn.opt.Model['input_size'] = train_x.shape[2]
        cnn.opt.cap = round(scaled_cap, 2)
        logger.info("---------cap归一化：{}".format(cnn.opt.cap))

        bp_model = cnn.training([train_x, train_y, valid_x, valid_y])

        args['params'] = json.dumps(args)
        args['descr'] = '测试'
        args['gen_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))

        insert_trained_model_into_mongo(bp_model, args)
        insert_scaler_model_into_mongo(scaled_train_bytes, scaled_target_bytes, args)
        success = 1
    except Exception as e:
        my_exception = traceback.format_exc()
        my_exception.replace("\n", "\t")
        result['msg'] = my_exception
    end_time = time.time()
    result['success'] = success
    result['args'] = args
    result['start_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(start_time))
    result['end_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(end_time))
    print("Program execution ends!")
    return result


if __name__ == "__main__":
    print("Program starts execution!")
    from waitress import serve
    serve(app, host="0.0.0.0", port=10113)
    # print("server start!")
    # args_dict = {"mongodb_database": 'david_test', 'scaler_table': 'j00083_scaler', 'model_name': 'bp1.0.test',
    # 'model_table': 'j00083_model', 'mongodb_read_table': 'j00083', 'col_time': 'dateTime',
    # 'features': 'speed10,direction10,speed30,direction30,speed50,direction50,speed70,direction70,speed90,direction90,speed110,direction110,speed150,direction150,speed170,direction170'}
    # args_dict['features'] = args_dict['features'].split(',')
    # arguments.update(args_dict)
    # dh = DataHandler(logger, arguments)
    # cnn = CNNHandler(logger)
    # opt = argparse.Namespace(**arguments)
    # opt.Model['input_size'] = len(opt.features)
    # train_data = get_data_from_mongo(args_dict)
    # train_x, valid_x, train_y, valid_y, scaled_train_bytes, scaled_target_bytes = dh.train_data_handler(train_data, opt)
    # cnn_model = cnn.training(opt, [train_x, train_y, valid_x, valid_y])
    #
    # args_dict['gen_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
    # args_dict['params'] = arguments
    # args_dict['descr'] = '测试'
    # insert_trained_model_into_mongo(cnn_model, args_dict)
    # insert_scaler_model_into_mongo(scaled_train_bytes, scaled_target_bytes, args_dict)