#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @FileName  :tf_lstm_train.py
# @Time      :2025/2/13 10:52
# @Author    :David
# @Company: shenyang JY
import json, copy
import numpy as np
from flask import Flask, request, jsonify
import traceback, uuid
import logging, argparse
from data_processing.data_operation.data_handler import DataHandler
import time, yaml, threading
from models_processing.model_tf.tf_lstm import TSHandler
from common.database_dml_koi import *
from common.logs import Log
logger = Log('tf_ts').logger
np.random.seed(42)  # NumPy随机种子
app = Flask('tf_lstm_train——service')

with app.app_context():
    current_dir = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(current_dir, 'lstm.yaml'), 'r', encoding='utf-8') as f:
        args = yaml.safe_load(f)

    dh = DataHandler(logger, args)
    ts = TSHandler(logger, args)

@app.before_request
def update_config():
    # ------------ 整理参数，整合请求参数 ------------
    args_dict = request.values.to_dict()
    args_dict['features'] = args_dict['features'].split(',')
    args.update(args_dict)
    opt = argparse.Namespace(**args)
    dh.opt = opt
    ts.opt = opt
    logger.info(args)

@app.route('/nn_lstm_training', methods=['POST'])
def model_training_bp():
    # 获取程序开始时间
    start_time = time.time()
    result = {}
    success = 0
    print("Program starts execution!")
    try:
        # ------------ 获取数据，预处理训练数据 ------------
        train_data = get_data_from_mongo(args)
        train_x, train_y, valid_x, valid_y, scaled_train_bytes, scaled_target_bytes, scaled_cap = dh.train_data_handler(train_data)
        # ------------ 训练模型，保存模型 ------------
        ts.opt.Model['input_size'] = train_x.shape[2]
        ts.opt.cap = round(scaled_cap, 2)
        ts_model = ts.training([train_x, train_y, valid_x, valid_y])

        args['params'] = json.dumps(args)
        args['descr'] = '测试'
        args['gen_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))

        insert_trained_model_into_mongo(ts_model, args)
        insert_scaler_model_into_mongo(scaled_train_bytes, scaled_target_bytes, args)
        success = 1
    except Exception as e:
        my_exception = traceback.format_exc()
        my_exception.replace("\n", "\t")
        result['msg'] = my_exception
    end_time = time.time()
    result['success'] = success
    result['args'] = args
    result['start_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(start_time))
    result['end_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(end_time))
    print("Program execution ends!")
    return result


if __name__ == "__main__":
    print("Program starts execution!")
    from waitress import serve
    serve(app, host="0.0.0.0", port=10115)
    print("server start!")
    # args_dict = {"mongodb_database": 'realtimeDq', 'scaler_table': 'j00600_scaler', 'model_name': 'lstm1',
    # 'model_table': 'j00600_model', 'mongodb_read_table': 'j00600', 'col_time': 'dateTime',
    # 'features': 'speed10,direction10,speed30,direction30,speed50,direction50,speed70,direction70,speed90,direction90,speed110,direction110,speed150,direction150,speed170,direction170'}
    # args_dict['features'] = args_dict['features'].split(',')
    # args.update(args_dict)
    # dh = DataHandler(logger, args)
    # ts = TSHandler(logger, args)
    # opt = argparse.Namespace(**args)
    # opt.Model['input_size'] = len(opt.features)
    # train_data = get_data_from_mongo(args_dict)
    # train_x, train_y, valid_x, valid_y, scaled_train_bytes, scaled_target_bytes = dh.train_data_handler(train_data)
    # ts_model = ts.training([train_x, train_y, valid_x, valid_y])
    #
    # args_dict['gen_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
    # args_dict['params'] = args
    # args_dict['descr'] = '测试'
    # insert_trained_model_into_mongo(ts_model, args_dict)
    # insert_scaler_model_into_mongo(scaled_train_bytes, scaled_target_bytes, args_dict)
